import { BehaviorSubject } from 'rxjs';
import { config } from '../config.js';
import { locationService } from './LocationService.js';

class SidecarService_EXPERIMENTAL {
  constructor(options) {
    this.localStorageKey = options.localStorageKey;
    let initialId = void 0;
    if (this.localStorageKey) {
      initialId = localStorage.getItem(this.localStorageKey) || void 0;
    }
    this._activePluginId = new BehaviorSubject(initialId);
    this._initialContext = new BehaviorSubject(void 0);
  }
  assertFeatureEnabled() {
    if (!config.featureToggles.appSidecar) {
      console.warn("The `appSidecar` feature toggle is not enabled, doing nothing.");
      return false;
    }
    return true;
  }
  /**
   * Get current app id of the app in sidecar. This is most probably provisional. In the future
   * this should be driven by URL addressing so that routing for the apps don't change. Useful just internally
   * to decide which app to render.
   *
   * @experimental
   */
  get activePluginIdObservable() {
    return this._activePluginId.asObservable();
  }
  /**
   * Get initial context which is whatever data was passed when calling the 'openApp' function. This is meant as
   * a way for the app to initialize it's state based on some context that is passed to it from the primary app.
   *
   * @experimental
   */
  get initialContextObservable() {
    return this._initialContext.asObservable();
  }
  // Get the current value of the subject, this is needed if we want the value immediately. For example if used in
  // hook in react with useObservable first render would return undefined even if the behaviourSubject has some
  // value which will be emitted in the next tick and thus next rerender.
  get initialContext() {
    return this._initialContext.getValue();
  }
  /**
   * @experimental
   */
  get activePluginId() {
    return this._activePluginId.getValue();
  }
  /**
   * Opens an app in a sidecar. You can also pass some context object that will be then available to the app.
   * @experimental
   */
  openApp(pluginId, context) {
    if (!this.assertFeatureEnabled()) {
      return;
    }
    if (this.localStorageKey) {
      localStorage.setItem(this.localStorageKey, pluginId);
    }
    this._activePluginId.next(pluginId);
    this._initialContext.next(context);
  }
  /**
   * @experimental
   */
  closeApp(pluginId) {
    if (!this.assertFeatureEnabled()) {
      return;
    }
    if (this._activePluginId.getValue() === pluginId) {
      if (this.localStorageKey) {
        localStorage.removeItem(this.localStorageKey);
      }
      this._activePluginId.next(void 0);
      this._initialContext.next(void 0);
    }
  }
  /**
   * This is mainly useful inside an app extensions which are executed outside of the main app context but can work
   * differently depending whether their app is currently rendered or not.
   * @experimental
   */
  isAppOpened(pluginId) {
    if (!this.assertFeatureEnabled()) {
      return false;
    }
    if (this._activePluginId.getValue() === pluginId || getMainAppPluginId() === pluginId) {
      return true;
    }
    return false;
  }
}
const sidecarServiceSingleton_EXPERIMENTAL = new SidecarService_EXPERIMENTAL({
  localStorageKey: "grafana.sidecar.activePluginId"
});
function getMainAppPluginId() {
  var _a;
  const { pathname } = locationService.getLocation();
  let mainApp = (_a = pathname.match(/\/a\/([^/]+)/)) == null ? void 0 : _a[1];
  if (!mainApp && pathname.match(/\/explore/)) {
    mainApp = "explore";
  }
  if (!mainApp && pathname.match(/\/d\//)) {
    mainApp = "dashboards";
  }
  return mainApp || "unknown";
}

export { SidecarService_EXPERIMENTAL, sidecarServiceSingleton_EXPERIMENTAL };
//# sourceMappingURL=SidecarService_EXPERIMENTAL.js.map
